import { createCommentVNode, defineComponent, getCurrentInstance, h, onUnmounted, watch } from 'vue'
import { VcComponentInternalInstance } from '@vue-cesium/utils/types'
import { usePrimitiveCollections } from '@vue-cesium/composables'
import cloneDeep from 'lodash/cloneDeep'
import differenceBy from 'lodash/differenceBy'
import {
  modelMatrix,
  debugShowBoundingVolume,
  blendOption,
  show,
  enableMouseEvent
} from '@vue-cesium/utils/cesium-props'
import { kebabCase } from '@vue-cesium/utils/util'
import { hSlot } from '@vue-cesium/utils/private/render'

export default defineComponent({
  name: 'VcCollectionPoint',
  props: {
    ...modelMatrix,
    ...debugShowBoundingVolume,
    ...blendOption,
    ...show,
    ...enableMouseEvent,
    points: {
      type: Array,
      default: () => []
    }
  },
  emits: ['beforeLoad', 'ready', 'destroyed'],
  setup (props, ctx) {
    // state
    const instance = getCurrentInstance() as VcComponentInternalInstance
    instance.cesiumClass = 'PointPrimitiveCollection'
    const primitiveCollectionsState = usePrimitiveCollections(props, ctx, instance)

    // watcher
    instance.alreadyListening.push('points')
    let unwatchFns = []
    unwatchFns.push(watch(
      () => cloneDeep(props.points),
      (newVal, oldVal) => {
        if (!instance.mounted) {
          return
        }
        const pointCollection = instance.cesiumObject as Cesium.PointPrimitiveCollection

        if (newVal.length === oldVal.length) {
          // 视为修改操作
          // Treated as modified
          const modifies = []
          for (let i = 0; i < newVal.length; i++) {
            const options = newVal[i]
            const oldOptions = oldVal[i]

            if (JSON.stringify(options) !== JSON.stringify(oldOptions)) {
              modifies.push({
                newOptions: options,
                oldOptions: oldOptions
              })
            }
          }

          modifies.forEach(modify => {
            const modifyPoint = pointCollection._pointPrimitives.find(v => v.id === modify.oldOptions.id)
            modifyPoint && Object.keys(modify.newOptions).forEach(prop => {
              if (modify.oldOptions[prop] !== modify.newOptions[prop]) {
                modifyPoint[prop] = primitiveCollectionsState.transformProp(prop, modify.newOptions[prop])
              }
            })
          })
        } else {
          const adds: any = differenceBy(newVal, oldVal, 'id')
          const deletes: any = differenceBy(oldVal, newVal, 'id')
          const deletePoints = []
          for (let i = 0; i < deletes.length; i++) {
            const deletePoint = pointCollection._pointPrimitives.find(v => v.id === deletes[i].id)
            deletePoint && deletePoints.push(deletePoint)
          }

          deletePoints.forEach(v => {
            pointCollection.remove(v)
          })

          for (let i = 0; i < adds.length; i++) {
            const pointOptions = newVal[i] as Cesium.Billboard
            pointOptions.id = Cesium.defined(pointOptions.id) ? pointOptions.id : Cesium.createGuid()
            const pointOptionsTransform = primitiveCollectionsState.transformProps(pointOptions)
            pointCollection.add(pointOptionsTransform)
          }
        }
      },
      {
        deep: true
      }
    ))
    // methods
    instance.createCesiumObject = async () => {
      const options = primitiveCollectionsState.transformProps(props)
      const pointCollection = new Cesium.PointPrimitiveCollection(options)

      for (let i = 0; i < props.points.length; i++) {
        const pointOptions = props.points[i] as Cesium.PointPrimitive
        pointOptions.id = Cesium.defined(pointOptions.id) ? pointOptions.id : Cesium.createGuid()
        const pointOptionsTransform = primitiveCollectionsState.transformProps(pointOptions)
        pointCollection.add(pointOptionsTransform)
      }
      return pointCollection
    }

    // life cycle
    onUnmounted(() => {
      unwatchFns.forEach(item => item())
      unwatchFns = []
    })

    return () => ctx.slots.default ? (
      h('i', {
        class: kebabCase(instance.proxy.$options.name),
        style: { display: 'none !important' }
      }, hSlot(ctx.slots.default))
    ) : createCommentVNode(kebabCase(instance.proxy.$options.name))
  }
})
